@@ -154,6 +154,15 @@ void uwsgi_master_check_mercy() {
 			}
 		}
 	}
+
+
+	struct uwsgi_spooler *us;
+	for (us = uwsgi.spoolers; us; us = us->next) {
+		if (us->pid > 0 && us->cursed_at && uwsgi_now() > us->no_mercy_at) {
+				uwsgi_log_verbose("spooler %d (pid: %d) is taking too much time to die...NO MERCY !!!\n", i + 1, us->pid);
+				kill(us->pid, SIGKILL);
+		}
+	}
 }
 
 
@@ -829,6 +838,18 @@ int master_loop(char **argv, char **environ) {
 					uwsgi_reload_workers();
 					continue;
 				}
+				touched = uwsgi_check_touches(uwsgi.touch_mules_reload);
+				if (touched) {
+					uwsgi_log_verbose("*** %s has been touched... mules reload !!! ***\n", touched);
+					uwsgi_reload_mules();
+					continue;
+				}
+				touched = uwsgi_check_touches(uwsgi.touch_spoolers_reload);
+				if (touched) {
+					uwsgi_log_verbose("*** %s has been touched... spoolers reload !!! ***\n", touched);
+					uwsgi_reload_spoolers();
+					continue;
+				}
 				touched = uwsgi_check_touches(uwsgi.touch_chain_reload);
 				if (touched) {
 					if (uwsgi.status.chain_reloading == 0) {
@@ -970,6 +991,21 @@ next:
 		// only to be safe :P
 		uwsgi.workers[thewid].harakiri = 0;
 
+
+		// first check failed app loading in need-app mode
+		if (WIFEXITED(waitpid_status) && WEXITSTATUS(waitpid_status) == UWSGI_FAILED_APP_CODE) {
+			if (uwsgi.lazy_apps && uwsgi.need_app) {
+				uwsgi_log("OOPS ! failed loading app in worker %d (pid %d)\n", thewid, (int) diedpid);
+				uwsgi_log_verbose("need-app requested, destroying the instance...\n");
+				uwsgi.status.dying_for_need_app = 1;
+				kill_them_all(0);
+				continue;
+			}
+			else {
+				uwsgi_log("OOPS ! failed loading app in worker %d (pid %d) :( trying again...\n", thewid, (int) diedpid);
+			}
+		}
+
 		// ok, if we are reloading or dying, just continue the master loop
 		// as soon as all of the workers have pid == 0, the action (exit, or reload) is triggered
 		if (uwsgi_instance_is_reloading || uwsgi_instance_is_dying) {
@@ -977,20 +1013,11 @@ next:
 				uwsgi.workers[thewid].cursed_at = uwsgi_now();
 			uwsgi_log("worker %d buried after %d seconds\n", thewid, (int) (uwsgi_now() - uwsgi.workers[thewid].cursed_at));
 			uwsgi.workers[thewid].cursed_at = 0;
+			// if we are stopping workers, just end here
 			continue;
 		}
 
-		// if we are stopping workers, just end here
-
-		if (WIFEXITED(waitpid_status) && WEXITSTATUS(waitpid_status) == UWSGI_FAILED_APP_CODE) {
-			uwsgi_log("OOPS ! failed loading app in worker %d (pid %d) :( trying again...\n", thewid, (int) diedpid);
-			if (uwsgi.lazy_apps && uwsgi.need_app) {
-				uwsgi_log_verbose("need-app requested, destroying the instance...\n");
-				kill_them_all(0);
-				continue;
-			}
-		}
-		else if (WIFEXITED(waitpid_status) && WEXITSTATUS(waitpid_status) == UWSGI_DE_HIJACKED_CODE) {
+		if (WIFEXITED(waitpid_status) && WEXITSTATUS(waitpid_status) == UWSGI_DE_HIJACKED_CODE) {
 			uwsgi_log("...restoring worker %d (pid: %d)...\n", thewid, (int) diedpid);
 		}
 		else if (WIFEXITED(waitpid_status) && WEXITSTATUS(waitpid_status) == UWSGI_EXCEPTION_CODE) {
@@ -1074,6 +1101,32 @@ void uwsgi_reload_workers() {
 	uwsgi_unblock_signal(SIGHUP);
 }
 
+void uwsgi_reload_mules() {
+	int i;
+
+	uwsgi_block_signal(SIGHUP);
+	for (i = 0; i <= uwsgi.mules_cnt; i++) {
+		if (uwsgi.mules[i].pid > 0) {
+			uwsgi_curse_mule(i, SIGHUP);
+		}
+	}
+	uwsgi_unblock_signal(SIGHUP);
+}
+
+void uwsgi_reload_spoolers() {
+	struct uwsgi_spooler *us;
+
+	uwsgi_block_signal(SIGHUP);
+	for (us = uwsgi.spoolers; us; us = us->next) {
+		if (us->pid > 0) {
+			kill(us->pid, SIGHUP);
+			us->cursed_at = uwsgi_now();
+			us->no_mercy_at = us->cursed_at + uwsgi.spooler_reload_mercy;
+		}
+	}
+	uwsgi_unblock_signal(SIGHUP);
+}
+
 void uwsgi_chain_reload() {
 	if (!uwsgi.status.chain_reloading) {
 		uwsgi_log_verbose("chain reload starting...\n");
